/*
  mcontrol - a MIDI controller software 

  midicMidiIn.C  -  MIDI Input functions.
  Copyright (C) 2003-2004 Josep Andreu (Holborn)
  Author: Josep Andreu

  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License (version 2) for more details.

  You should have received a copy of the GNU General Public License (version2)
  along with this program; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/


#include "global.h"


void
MCONTROL::midievents (int keIN)
{
  snd_seq_event_t *midievent;
  midievent = NULL;
  snd_seq_event_input (MidiInPuerto[keIN].midi_in, &midievent);
  if (midievent == NULL) return;
  if ((midievent->type == SND_SEQ_EVENT_NOTEON)
      || (midievent->type == SND_SEQ_EVENT_NOTEOFF))
    {
      unsigned int cmdchan = midievent->data.note.channel;
      unsigned int cmdvelo = midievent->data.note.velocity;
      
      vumi[keIN].ulvum = vumi[keIN].vumvum;
      vumi[keIN].vumvum = cmdvelo;
      
      snd_seq_ev_set_subs (midievent);
      snd_seq_ev_set_direct (midievent);
      
      int cmdv = cmdvelo;
      cmdv += velo;
      if (cmdv > 127) cmdv = 127;
      if (cmdv < 0) cmdv = 0;
      midievent->data.note.velocity = cmdv;
      midievent->data.note.channel = fCH(cmdchan);      
		    
      snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);

      vumo[1].uvo = midievent->data.note.velocity;


    };


  if (midievent->type == SND_SEQ_EVENT_CHANPRESS)
    {

      
      unsigned int cmdchan = midievent->data.control.channel;
      unsigned int cmdvalue = midievent->data.control.value;
      vumi[keIN].ulvum = vumi[keIN].vumvum;
      vumi[keIN].vumvum = cmdvalue;
      if (Bypass[3]==0) procesa(3,fCH(cmdchan),cmdvalue);
      else
      {
      snd_seq_ev_set_subs (midievent);
      snd_seq_ev_set_direct (midievent);
      
      midievent->data.note.channel = fCH(cmdchan);
      snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
      vumo[1].uvo = midievent->data.control.value;
      }		    
		  
    };



  if (midievent->type == SND_SEQ_EVENT_PITCHBEND)
    {
      unsigned int traduccion; 
      unsigned int cmdchan = midievent->data.control.channel;
      unsigned int cmdvalue = midievent->data.control.value;
      
      traduccion = PB2CC(cmdvalue);
      
      vumi[keIN].ulvum = vumi[keIN].vumvum;
      vumi[keIN].vumvum = traduccion;

      if (Bypass[2]==0) procesa(2,fCH(cmdchan),traduccion);
      else
      {
      snd_seq_ev_set_subs (midievent);
      snd_seq_ev_set_direct (midievent);
      
      midievent->data.note.channel = fCH(cmdchan);
      
      snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
      vumo[1].uvo = midievent->data.control.value;
      }		    
		  
    };


  if  (midievent->type == SND_SEQ_EVENT_CONTROLLER)
    {

      unsigned int cmdchan = midievent->data.control.channel;
      unsigned int cmdparam = midievent->data.control.param;
      unsigned int cmdvalue = midievent->data.control.value;

      vumi[keIN].ulvum = vumi[keIN].vumvum;
      vumi[keIN].vumvum = cmdvalue;

      switch (cmdparam)
         {
           case 1:
                   if (Bypass[1]==0) procesa(1,fCH(cmdchan),cmdvalue);
                   else
                   {
                   snd_seq_ev_set_subs (midievent);
                   snd_seq_ev_set_direct (midievent);
                   
                   midievent->data.note.channel = fCH(cmdchan);

                   snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
                   vumo[1].uvo = midievent->data.control.value;
                   }  
                   break;
           case 2:
                   if (Bypass[4]==0) procesa(4,fCH(cmdchan),cmdvalue);
                   else
                   {
                   snd_seq_ev_set_subs (midievent);
                   snd_seq_ev_set_direct (midievent);
                   
                   midievent->data.note.channel = fCH(cmdchan);
      
                   snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
                   vumo[1].uvo = midievent->data.control.value;
                   }
                   break;
           case 4:
                   if (Bypass[5]==0) procesa(5,fCH(cmdchan),cmdvalue);
                   else
                   {
                   snd_seq_ev_set_subs (midievent);
                   snd_seq_ev_set_direct (midievent);
                   
                   midievent->data.note.channel = fCH(cmdchan);
      
                   snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
                   vumo[1].uvo = midievent->data.control.value;
                   }
                   
                   break;
           default: 

                   snd_seq_ev_set_subs (midievent);
                   snd_seq_ev_set_direct (midievent);
                   
                   midievent->data.note.channel = fCH(cmdchan);
      
                   snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
                   vumo[1].uvo = midievent->data.control.value;
           	break;
        }	    
		  
    };

  if  (midievent->type == SND_SEQ_EVENT_PGMCHANGE)
  
    {
   
   
     unsigned int cmdchan = midievent->data.control.channel;
     unsigned int cmdvalue = midievent->data.control.value;
     vumi[keIN].ulvum = vumi[keIN].vumvum;
     vumi[keIN].vumvum = cmdvalue;
              

     if ((cmdchan == (propchan-1)) && (cmdvalue > 0) && (cmdvalue < 45)) 
           
                  {   programa = cmdvalue;
                      return;
                  }
                                                 

     snd_seq_ev_set_subs (midievent);
     snd_seq_ev_set_direct (midievent);
     
     midievent->data.note.channel = fCH(cmdchan);
           
     snd_seq_event_output_direct (MidiOutPuerto[1].midi_out, midievent);
     vumo[1].uvo = midievent->data.control.value; 
    
    
    }
    


};


